@@ -25,7 +25,7 @@
 #include "npsolswitch.h"
 #include "EnableWarnings.h"
 
-struct omxGlobal *Global = NULL;
+struct std::unique_ptr<omxGlobal> Global;
 static bool mxLogEnabled = false;
 
 SEXP enableMxLog()
@@ -271,6 +271,8 @@ omxGlobal::omxGlobal()
 	// is not instrumented then false positives can result,
 	// https://github.com/google/sanitizers/wiki/AddressSanitizerContainerOverflow
 	checkpointColnames.reserve(100);
+
+  globalState = std::make_unique<omxState>();
 }
 
 void omxGlobal::setDefaultGradientAlgo()
@@ -378,6 +380,7 @@ void omxGlobal::unpackConfidenceIntervals(omxState *currentState)
 			} else if (ci->cmpBoundAndType(**iter)) {
 				Rf_warning("Different confidence intervals '%s' and '%s' refer to the same thing",
 					   ci->name.c_str(), (*iter)->name.c_str());
+        delete ci;
 			}
 			continue;
 		}
@@ -584,8 +587,7 @@ omxState::~omxState()
 	}
 
 	for(size_t ax = 0; ax < algebraList.size(); ax++) {
-		// free argument tree
-		omxFreeMatrix(algebraList[ax]);
+		algebraList[ax]->disconnect();
 	}
 
 	for(size_t ax = 0; ax < algebraList.size(); ax++) {
@@ -607,21 +609,23 @@ omxState::~omxState()
 
 omxGlobal::~omxGlobal()
 {
+	// Data are not modified and not copied. The same memory
+	// is shared across all instances of state.
+	// NOTE: This may need to change for MxDataDynamic
+	for(size_t dx = 0; dx < globalState->dataList.size(); dx++) {
+		omxFreeData(globalState->dataList[dx]);
+	}
+
 	if (!previousReport.empty()) {
 		std::string empty;
 		reportProgressStr(empty);
 	}
 	if (topFc) {
-		omxState *state = topFc->state;
 		delete topFc;
-		delete state;
 	}
 	for (size_t cx=0; cx < intervalList.size(); ++cx) {
 		delete intervalList[cx];
 	}
-	for (size_t cx=0; cx < computeList.size(); ++cx) {
-		delete computeList[cx];
-	}
 	for (size_t cx=0; cx < checkpointList.size(); ++cx) {
 		delete checkpointList[cx];
 	}
